@@ -450,6 +450,27 @@ async def test_FinePerformanceMetrics_no_spans(c, s, a, b):
     assert sorted(cl.visible_activities) == ["memory-read"]
 
 
+@gen_cluster(client=True)
+async def test_FinePerformanceMetrics_shuffle(c, s, a, b):
+    da = pytest.importorskip("dask.array")
+
+    x = da.random.random((4, 4), chunks=(1, -1))
+    x = x.rechunk((-1, 1), method="p2p")
+    x = x.sum()
+    await c.compute(x)
+    await a.heartbeat()
+    await b.heartbeat()
+
+    cl = FinePerformanceMetrics(s)
+    cl.update()
+    # execute metrics from shuffle
+    assert "shuffle-barrier" in cl.visible_functions
+    assert "p2p-shard-partition-cpu" in cl.visible_activities
+    # shuffle metrics have been filtered out
+    assert "background-comms" not in cl.visible_functions
+    assert "p2p-partition-cpu" not in cl.visible_activities
+
+
 @gen_cluster(client=True)
 async def test_ClusterMemory(c, s, a, b):
     cl = ClusterMemory(s)
@@ -989,6 +1010,7 @@ async def test_TaskGroupGraph_arrows(c, s, a, b):
     assert not any(tgg.arrows_source.data.values())
 
 
+@pytest.mark.skipif(sys.version_info.minor == 11, reason="Profiler disabled")
 @gen_cluster(
     client=True,
     config={
@@ -1122,6 +1144,7 @@ async def test_https_support(c, s, a, b):
 
     ctx = ssl.create_default_context()
     ctx.load_verify_locations(get_cert("tls-ca-cert.pem"))
+    ctx.verify_flags &= ~ssl.VERIFY_X509_STRICT
 
     http_client = AsyncHTTPClient()
     response = await http_client.fetch(
@@ -1265,7 +1288,6 @@ async def test_compute_per_key(c, s, a, b):
 
     da = pytest.importorskip("dask.array")
     x = (da.ones((20, 20), chunks=(10, 10)) + 1).persist(optimize_graph=False)
-
     await x
     y = await dask.delayed(inc)(1).persist()
     z = (x + x.T) - x.mean(axis=0)
@@ -1273,14 +1295,18 @@ async def test_compute_per_key(c, s, a, b):
     await c.compute(zsum)
 
     mbk.update()
+
+    # Keep only times which are 2% of max or greater.
+    # This means that the list of names is not stable (but max time is always preserved)
+    assert mbk.compute_source.data["names"]
+    assert set(mbk.compute_source.data["names"]).issubset(s.task_prefixes)
+    assert "angles" in mbk.compute_source.data
+
     http_client = AsyncHTTPClient()
     response = await http_client.fetch(
         "http://localhost:%d/individual-compute-time-per-key" % s.http_server.port
     )
     assert response.code == 200
-    assert ("sum-aggregate") in mbk.compute_source.data["names"]
-    assert ("add") in mbk.compute_source.data["names"]
-    assert "angles" in mbk.compute_source.data.keys()
 
 
 @gen_cluster(scheduler_kwargs={"http_prefix": "foo-bar", "dashboard": True})
@@ -1321,13 +1347,14 @@ async def test_shuffling(c, s, a, b):
         dtypes={"x": float, "y": float},
         freq="10 s",
     )
-    df2 = dd.shuffle.shuffle(df, "x", shuffle="p2p").persist()
+    with dask.config.set({"dataframe.shuffle.method": "p2p"}):
+        df2 = df.shuffle("x").persist()
     start = time()
     while not ss.source.data["comm_written"]:
+        await asyncio.gather(*[a.heartbeat(), b.heartbeat()])
         ss.update()
-        await asyncio.sleep(0)
-        assert time() < start + 5
-    await df2
+        await asyncio.sleep(0.01)
+        assert time() < start + 10
 
 
 @gen_cluster(client=True, scheduler_kwargs={"dashboard": True}, timeout=60)
