@@ -184,35 +184,44 @@ def test_transports_tcp_port(loop):
 
 
 def test_cores(loop):
-    with LocalCluster(
-        n_workers=2,
-        scheduler_port=0,
-        silence_logs=False,
-        dashboard_address=":0",
-        processes=False,
-        loop=loop,
-    ) as cluster, Client(cluster.scheduler_address, loop=loop) as client:
+    with (
+        LocalCluster(
+            n_workers=2,
+            scheduler_port=0,
+            silence_logs=False,
+            dashboard_address=":0",
+            processes=False,
+            loop=loop,
+        ) as cluster,
+        Client(cluster.scheduler_address, loop=loop) as client,
+    ):
         client.scheduler_info()
         assert len(client.nthreads()) == 2
 
 
 def test_submit(loop):
-    with LocalCluster(
-        n_workers=2,
-        scheduler_port=0,
-        silence_logs=False,
-        dashboard_address=":0",
-        processes=False,
-        loop=loop,
-    ) as cluster, Client(cluster.scheduler_address, loop=loop) as client:
+    with (
+        LocalCluster(
+            n_workers=2,
+            scheduler_port=0,
+            silence_logs=False,
+            dashboard_address=":0",
+            processes=False,
+            loop=loop,
+        ) as cluster,
+        Client(cluster.scheduler_address, loop=loop) as client,
+    ):
         future = client.submit(lambda x: x + 1, 1)
         assert future.result() == 2
 
 
 def test_context_manager(loop):
-    with LocalCluster(
-        silence_logs=False, dashboard_address=":0", processes=False, loop=loop
-    ) as c, Client(c) as e:
+    with (
+        LocalCluster(
+            silence_logs=False, dashboard_address=":0", processes=False, loop=loop
+        ) as c,
+        Client(c) as e,
+    ):
         assert e.nthreads()
 
 
@@ -829,14 +838,17 @@ async def test_scale_retires_workers():
         def scale_down(self, *args, **kwargs):
             pass
 
-    async with MyCluster(
-        n_workers=0,
-        processes=False,
-        silence_logs=False,
-        dashboard_address=":0",
-        loop=None,
-        asynchronous=True,
-    ) as cluster, Client(cluster, asynchronous=True) as c:
+    async with (
+        MyCluster(
+            n_workers=0,
+            processes=False,
+            silence_logs=False,
+            dashboard_address=":0",
+            loop=None,
+            asynchronous=True,
+        ) as cluster,
+        Client(cluster, asynchronous=True) as c,
+    ):
         assert not cluster.workers
 
         await cluster.scale(2)
@@ -1054,10 +1066,14 @@ async def test_threads_per_worker_set_to_0():
         Warning, match="Setting `threads_per_worker` to 0 has been deprecated."
     ):
         async with LocalCluster(
-            n_workers=2, processes=False, threads_per_worker=0, asynchronous=True
+            n_workers=2,
+            processes=False,
+            threads_per_worker=0,
+            asynchronous=True,
+            dashboard_address=":0",
         ) as cluster:
             assert len(cluster.workers) == 2
-            assert all(w.nthreads < CPU_COUNT for w in cluster.workers.values())
+            assert all(w.state.nthreads < CPU_COUNT for w in cluster.workers.values())
 
 
 @pytest.mark.parametrize("temporary", [True, False])
@@ -1158,7 +1174,10 @@ async def test_local_cluster_redundant_kwarg(nanny):
 @gen_test()
 async def test_cluster_info_sync():
     async with LocalCluster(
-        processes=False, asynchronous=True, scheduler_sync_interval="1ms"
+        processes=False,
+        asynchronous=True,
+        scheduler_sync_interval="1ms",
+        dashboard_address=":0",
     ) as cluster:
         assert cluster._cluster_info["name"] == cluster.name
 
@@ -1185,7 +1204,10 @@ async def test_cluster_info_sync():
 @gen_test()
 async def test_cluster_info_sync_is_robust_to_network_blips(monkeypatch):
     async with LocalCluster(
-        processes=False, asynchronous=True, scheduler_sync_interval="1ms"
+        processes=False,
+        asynchronous=True,
+        scheduler_sync_interval="1ms",
+        dashboard_address=":0",
     ) as cluster:
         assert cluster._cluster_info["name"] == cluster.name
 
@@ -1223,7 +1245,9 @@ async def test_cluster_info_sync_is_robust_to_network_blips(monkeypatch):
 @gen_test()
 async def test_cluster_host_used_throughout_cluster(host, use_nanny):
     """Ensure that the `host` kwarg is propagated through scheduler, nanny, and workers"""
-    async with LocalCluster(host=host, asynchronous=True) as cluster:
+    async with LocalCluster(
+        host=host, asynchronous=True, dashboard_address=":0"
+    ) as cluster:
         url = urlparse(cluster.scheduler_address)
         assert url.hostname == "127.0.0.1"
         for worker in cluster.workers.values():
@@ -1237,7 +1261,9 @@ async def test_cluster_host_used_throughout_cluster(host, use_nanny):
 
 @gen_test()
 async def test_connect_to_closed_cluster():
-    async with LocalCluster(processes=False, asynchronous=True) as cluster:
+    async with LocalCluster(
+        processes=False, asynchronous=True, dashboard_address=":0"
+    ) as cluster:
         async with Client(cluster, asynchronous=True) as c1:
             assert await c1.submit(inc, 1) == 2
 
