@@ -7,6 +7,7 @@ from collections.abc import (
     Callable,
     Collection,
     Generator,
+    Hashable,
     Iterable,
     Iterator,
     Sequence,
@@ -20,24 +21,27 @@ import toolz
 from tornado.ioloop import IOLoop
 
 import dask
-from dask.base import tokenize
+from dask._task_spec import GraphNode, Task, TaskRef
 from dask.highlevelgraph import HighLevelGraph
 from dask.layers import Layer
+from dask.tokenize import tokenize
 from dask.typing import Key
+from dask.utils import is_dataframe_like
 
 from distributed.core import PooledRPCCall
-from distributed.exceptions import Reschedule
+from distributed.metrics import context_meter
 from distributed.shuffle._arrow import (
+    buffers_to_table,
     check_dtype_support,
     check_minimal_arrow_version,
     convert_shards,
     deserialize_table,
-    list_of_buffers_to_table,
     read_from_disk,
     serialize_table,
 )
 from distributed.shuffle._core import (
     NDIndex,
+    P2PBarrierTask,
     ShuffleId,
     ShuffleRun,
     ShuffleSpec,
@@ -45,7 +49,9 @@ from distributed.shuffle._core import (
     get_worker_plugin,
     handle_transfer_errors,
     handle_unpack_errors,
+    p2p_barrier,
 )
+from distributed.shuffle._exceptions import DataUnavailable
 from distributed.shuffle._limiter import ResourceLimiter
 from distributed.shuffle._worker_plugin import ShuffleWorkerPlugin
 from distributed.sizeof import sizeof
@@ -65,24 +71,12 @@ def shuffle_transfer(
     input: pd.DataFrame,
     id: ShuffleId,
     input_partition: int,
-    npartitions: int,
-    column: str,
-    meta: pd.DataFrame,
-    parts_out: set[int],
-    disk: bool,
 ) -> int:
     with handle_transfer_errors(id):
         return get_worker_plugin().add_partition(
             input,
             input_partition,
-            spec=DataFrameShuffleSpec(
-                id=id,
-                npartitions=npartitions,
-                column=column,
-                meta=meta,
-                parts_out=parts_out,
-                disk=disk,
-            ),
+            id,
         )
 
 
@@ -95,15 +89,6 @@ def shuffle_unpack(
         )
 
 
-def shuffle_barrier(id: ShuffleId, run_ids: list[int]) -> int:
-    try:
-        return get_worker_plugin().barrier(id, run_ids)
-    except Reschedule as e:
-        raise e
-    except Exception as e:
-        raise RuntimeError(f"shuffle_barrier failed during shuffle {id}") from e
-
-
 def rearrange_by_column_p2p(
     df: DataFrame,
     column: str,
@@ -129,7 +114,7 @@ def rearrange_by_column_p2p(
         )
 
     name = f"shuffle_p2p-{token}"
-    disk: bool = dask.config.get("distributed.p2p.disk")
+    disk: bool = dask.config.get("distributed.p2p.storage.disk")
 
     layer = P2PShuffleLayer(
         name,
@@ -148,7 +133,7 @@ def rearrange_by_column_p2p(
     )
 
 
-_T_LowLevelGraph: TypeAlias = dict[Key, tuple]
+_T_LowLevelGraph: TypeAlias = dict[Key, GraphNode]
 
 
 class P2PShuffleLayer(Layer):
@@ -159,7 +144,8 @@ class P2PShuffleLayer(Layer):
     name_input: str
     meta_input: pd.DataFrame
     disk: bool
-    parts_out: set[int]
+    parts_out: tuple[int, ...]
+    drop_column: bool
 
     def __init__(
         self,
@@ -172,6 +158,7 @@ class P2PShuffleLayer(Layer):
         disk: bool,
         parts_out: Iterable[int] | None = None,
         annotations: dict | None = None,
+        drop_column: bool = False,
     ):
         check_minimal_arrow_version()
         self.name = name
@@ -181,10 +168,11 @@ class P2PShuffleLayer(Layer):
         self.meta_input = meta_input
         self.disk = disk
         if parts_out:
-            self.parts_out = set(parts_out)
+            self.parts_out = tuple(parts_out)
         else:
-            self.parts_out = set(range(self.npartitions))
+            self.parts_out = tuple(range(self.npartitions))
         self.npartitions_input = npartitions_input
+        self.drop_column = drop_column
         super().__init__(annotations=annotations)
 
     def __repr__(self) -> str:
@@ -210,7 +198,7 @@ class P2PShuffleLayer(Layer):
             self._cached_dict = dsk
         return self._cached_dict
 
-    def __getitem__(self, key: Key) -> tuple:
+    def __getitem__(self, key: Key) -> GraphNode:
         return self._dict[key]
 
     def __iter__(self) -> Iterator[Key]:
@@ -267,34 +255,49 @@ class P2PShuffleLayer(Layer):
 
     def _construct_graph(self) -> _T_LowLevelGraph:
         token = tokenize(self.name_input, self.column, self.npartitions, self.parts_out)
+        shuffle_id = ShuffleId(token)
         dsk: _T_LowLevelGraph = {}
-        _barrier_key = barrier_key(ShuffleId(token))
+        _barrier_key = barrier_key(shuffle_id)
         name = "shuffle-transfer-" + token
         transfer_keys = list()
         for i in range(self.npartitions_input):
-            transfer_keys.append((name, i))
-            dsk[(name, i)] = (
+            t = Task(
+                (name, i),
                 shuffle_transfer,
-                (self.name_input, i),
+                TaskRef((self.name_input, i)),
                 token,
                 i,
-                self.npartitions,
-                self.column,
-                self.meta_input,
-                self.parts_out,
-                self.disk,
             )
+            dsk[t.key] = t
+            transfer_keys.append(t.ref())
 
-        dsk[_barrier_key] = (shuffle_barrier, token, transfer_keys)
+        barrier = P2PBarrierTask(
+            _barrier_key,
+            p2p_barrier,
+            token,
+            *transfer_keys,
+            spec=DataFrameShuffleSpec(
+                id=shuffle_id,
+                npartitions=self.npartitions,
+                column=self.column,
+                meta=self.meta_input,
+                parts_out=self.parts_out,
+                disk=self.disk,
+                drop_column=self.drop_column,
+            ),
+        )
+        dsk[barrier.key] = barrier
 
         name = self.name
         for part_out in self.parts_out:
-            dsk[(name, part_out)] = (
+            t = Task(
+                (name, part_out),
                 shuffle_unpack,
                 token,
                 part_out,
-                _barrier_key,
+                barrier.ref(),
             )
+            dsk[t.key] = t
         return dsk
 
 
@@ -313,8 +316,7 @@ def split_by_worker(
 
     # (cudf support) Avoid pd.Series
     constructor = df._constructor_sliced
-    assert isinstance(constructor, type)
-    worker_for = constructor(worker_for)
+    worker_for = constructor(worker_for)  # type: ignore
     df = df.merge(
         right=worker_for.cat.codes.rename("_worker"),
         left_on=column,
@@ -351,7 +353,9 @@ def split_by_worker(
     return out
 
 
-def split_by_partition(t: pa.Table, column: str) -> dict[int, pa.Table]:
+def split_by_partition(
+    t: pa.Table, column: str, drop_column: bool
+) -> dict[int, pa.Table]:
     """
     Split data into many arrow batches, partitioned by final partition
     """
@@ -362,6 +366,8 @@ def split_by_partition(t: pa.Table, column: str) -> dict[int, pa.Table]:
     t = t.sort_by(column)
 
     partition = np.asarray(t[column])
+    if drop_column:
+        t = t.drop([column])
     splits = np.where(partition[1:] != partition[:-1])[0] + 1
     splits = np.concatenate([[0], splits])
 
@@ -396,6 +402,8 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
         A unique `ShuffleID` this belongs to.
     run_id:
         A unique identifier of the specific execution of the shuffle this belongs to.
+    span_id:
+        Span identifier; see :doc:`spans`
     local_address:
         The local address this Shuffle can be contacted by using `rpc`.
     directory:
@@ -405,8 +413,11 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
     rpc:
         A callable returning a PooledRPCCall to contact other Shuffle instances.
         Typically a ConnectionPool.
+    digest_metric:
+        A callable to ingest a performance metric.
+        Typically Server.digest_metric.
     scheduler:
-        A PooledRPCCall to to contact the scheduler.
+        A PooledRPCCall to contact the scheduler.
     memory_limiter_disk:
     memory_limiter_comm:
         A ``ResourceLimiter`` limiting the total amount of memory used in either
@@ -417,6 +428,7 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
     meta: pd.DataFrame
     partitions_of: dict[str, list[int]]
     worker_for: pd.Series
+    drop_column: bool
 
     def __init__(
         self,
@@ -425,14 +437,17 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
         meta: pd.DataFrame,
         id: ShuffleId,
         run_id: int,
+        span_id: str | None,
         local_address: str,
         directory: str,
         executor: ThreadPoolExecutor,
         rpc: Callable[[str], PooledRPCCall],
+        digest_metric: Callable[[Hashable, float], None],
         scheduler: PooledRPCCall,
         memory_limiter_disk: ResourceLimiter,
         memory_limiter_comms: ResourceLimiter,
         disk: bool,
+        drop_column: bool,
         loop: IOLoop,
     ):
         import pandas as pd
@@ -440,10 +455,12 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
         super().__init__(
             id=id,
             run_id=run_id,
+            span_id=span_id,
             local_address=local_address,
             directory=directory,
             executor=executor,
             rpc=rpc,
+            digest_metric=digest_metric,
             scheduler=scheduler,
             memory_limiter_comms=memory_limiter_comms,
             memory_limiter_disk=memory_limiter_disk,
@@ -457,6 +474,7 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
             partitions_of[addr].append(part)
         self.partitions_of = dict(partitions_of)
         self.worker_for = pd.Series(worker_for, name="_workers").astype("category")
+        self.drop_column = drop_column
 
     async def _receive(self, data: list[tuple[int, bytes]]) -> None:
         self.raise_if_closed()
@@ -464,7 +482,7 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
         filtered = []
         for d in data:
             if d[0] not in self.received:
-                filtered.append(d[1])
+                filtered.append(d)
                 self.received.add(d[0])
                 self.total_recvd += sizeof(d)
         del data
@@ -478,9 +496,11 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
             self._exception = e
             raise
 
-    def _repartition_buffers(self, data: list[bytes]) -> dict[NDIndex, bytes]:
-        table = list_of_buffers_to_table(data)
-        groups = split_by_partition(table, self.column)
+    def _repartition_buffers(
+        self, data: list[tuple[int, bytes]]
+    ) -> dict[NDIndex, bytes]:
+        table = buffers_to_table(data)
+        groups = split_by_partition(table, self.column, self.drop_column)
         assert len(table) == sum(map(len, groups.values()))
         del data
         return {(k,): serialize_table(v) for k, v in groups.items()}
@@ -498,6 +518,11 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
             self.worker_for,
         )
         out = {k: (partition_id, serialize_table(t)) for k, t in out.items()}
+
+        nbytes = sum(len(b) for _, b in out.values())
+        context_meter.digest_metric("p2p-shards", nbytes, "bytes")
+        context_meter.digest_metric("p2p-shards", len(out), "count")
+
         return out
 
     def _get_output_partition(
@@ -508,9 +533,12 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
     ) -> pd.DataFrame:
         try:
             data = self._read_from_disk((partition_id,))
-            return convert_shards(data, self.meta)
-        except KeyError:
-            return self.meta.copy()
+            return convert_shards(data, self.meta, self.column, self.drop_column)
+        except DataUnavailable:
+            result = self.meta.copy()
+            if self.drop_column:
+                result = self.meta.drop(columns=self.column)
+            return result
 
     def _get_assigned_worker(self, id: int) -> str:
         return self.worker_for[id]
@@ -521,23 +549,38 @@ class DataFrameShuffleRun(ShuffleRun[int, "pd.DataFrame"]):
     def deserialize(self, buffer: Any) -> Any:
         return deserialize_table(buffer)
 
+    def validate_data(self, data: pd.DataFrame) -> None:
+        if not is_dataframe_like(data):
+            raise TypeError(f"Expected {data=} to be a DataFrame, got {type(data)}.")
+        if set(data.columns) != set(self.meta.columns):
+            raise ValueError(f"Expected {self.meta.columns=} to match {data.columns=}.")
+
 
 @dataclass(frozen=True)
 class DataFrameShuffleSpec(ShuffleSpec[int]):
     npartitions: int
     column: str
     meta: pd.DataFrame
-    parts_out: set[int]
+    parts_out: Sequence[int] | int
+    drop_column: bool
 
     @property
-    def output_partitions(self) -> Generator[int, None, None]:
-        yield from self.parts_out
+    def output_partitions(self) -> Generator[int]:
+        parts_out = self.parts_out
+        if isinstance(parts_out, int):
+            parts_out = range(parts_out)
+
+        yield from parts_out
 
     def pick_worker(self, partition: int, workers: Sequence[str]) -> str:
         return _get_worker_for_range_sharding(self.npartitions, partition, workers)
 
     def create_run_on_worker(
-        self, run_id: int, worker_for: dict[int, str], plugin: ShuffleWorkerPlugin
+        self,
+        run_id: int,
+        span_id: str | None,
+        worker_for: dict[int, str],
+        plugin: ShuffleWorkerPlugin,
     ) -> ShuffleRun:
         return DataFrameShuffleRun(
             column=self.column,
@@ -545,6 +588,7 @@ class DataFrameShuffleSpec(ShuffleSpec[int]):
             worker_for=worker_for,
             id=self.id,
             run_id=run_id,
+            span_id=span_id,
             directory=os.path.join(
                 plugin.worker.local_directory,
                 f"shuffle-{self.id}-{run_id}",
@@ -552,12 +596,14 @@ class DataFrameShuffleSpec(ShuffleSpec[int]):
             executor=plugin._executor,
             local_address=plugin.worker.address,
             rpc=plugin.worker.rpc,
+            digest_metric=plugin.worker.digest_metric,
             scheduler=plugin.worker.scheduler,
-            memory_limiter_disk=plugin.memory_limiter_disk
-            if self.disk
-            else ResourceLimiter(None),
+            memory_limiter_disk=(
+                plugin.memory_limiter_disk if self.disk else ResourceLimiter(None)
+            ),
             memory_limiter_comms=plugin.memory_limiter_comms,
             disk=self.disk,
+            drop_column=self.drop_column,
             loop=plugin.worker.loop,
         )
 
