@@ -4,14 +4,17 @@ import asyncio
 import math
 import random
 import warnings
+from collections import defaultdict
 
 import pytest
-from packaging.version import parse as parse_version
+
+from distributed.diagnostics.plugin import SchedulerPlugin
 
 np = pytest.importorskip("numpy")
 da = pytest.importorskip("dask.array")
 
 from concurrent.futures import ThreadPoolExecutor
+from itertools import product
 
 from tornado.ioloop import IOLoop
 
@@ -19,6 +22,8 @@ import dask
 from dask.array.core import concatenate3
 from dask.array.rechunk import normalize_chunks, rechunk
 from dask.array.utils import assert_eq
+from dask.base import flatten
+from dask.utils import key_split
 
 from distributed import Event
 from distributed.protocol.utils_test import get_host_array
@@ -28,12 +33,11 @@ from distributed.shuffle._rechunk import (
     ArrayRechunkRun,
     ArrayRechunkSpec,
     Split,
+    _calculate_prechunking,
     split_axes,
 )
 from distributed.shuffle.tests.utils import AbstractShuffleTestPool
-from distributed.utils_test import gen_cluster, gen_test, raises_with_cause
-
-NUMPY_GE_124 = parse_version(np.__version__) >= parse_version("1.24")
+from distributed.utils_test import async_poll_for, gen_cluster, gen_test
 
 
 class ArrayRechunkTestPool(AbstractShuffleTestPool):
@@ -68,9 +72,11 @@ class ArrayRechunkTestPool(AbstractShuffleTestPool):
             directory=directory / name,
             id=ShuffleId(name),
             run_id=next(AbstractShuffleTestPool._shuffle_run_id_iterator),
+            span_id=None,
             local_address=name,
             executor=self._executor,
             rpc=self,
+            digest_metric=lambda name, value: None,
             scheduler=self,
             memory_limiter_disk=ResourceLimiter(10000000),
             memory_limiter_comms=ResourceLimiter(10000000),
@@ -81,9 +87,6 @@ class ArrayRechunkTestPool(AbstractShuffleTestPool):
         return s
 
 
-from itertools import product
-
-
 @pytest.mark.parametrize("n_workers", [1, 10])
 @pytest.mark.parametrize("barrier_first_worker", [True, False])
 @pytest.mark.parametrize("disk", [True, False])
@@ -172,7 +175,7 @@ async def test_lowlevel_rechunk(tmp_path, n_workers, barrier_first_worker, disk)
         np.testing.assert_array_equal(
             concatenate3(old_cs.tolist()),
             concatenate3(all_chunks.tolist()),
-            **({"strict": True} if NUMPY_GE_124 else {}),
+            strict=True,
         )
 
 
@@ -186,22 +189,143 @@ async def test_rechunk_configuration(c, s, *ws, config_value, keyword):
     --------
     dask.array.tests.test_rechunk.test_rechunk_1d
     """
-    a = np.random.default_rng().uniform(0, 1, 30)
-    x = da.from_array(a, chunks=((10,) * 3,))
-    new = ((6,) * 5,)
+    a = np.random.default_rng().uniform(0, 1, 100).reshape((10, 10))
+    x = da.from_array(a, chunks=(10, 1))
+    new = ((1,) * 10, (10,))
     config = {"array.rechunk.method": config_value} if config_value is not None else {}
     with dask.config.set(config):
         x2 = rechunk(x, chunks=new, method=keyword)
     expected_algorithm = keyword if keyword is not None else config_value
     if expected_algorithm == "p2p":
-        assert all(key[0].startswith("rechunk-p2p") for key in x2.__dask_keys__())
+        assert all(key[0][0].startswith("rechunk-p2p") for key in x2.__dask_keys__())
+    elif expected_algorithm == "tasks":
+        assert not any(
+            key[0][0].startswith("rechunk-p2p") for key in x2.__dask_keys__()
+        )
+    # Neither is specified, so we choose the best one (see test_rechunk_heuristic for a full test of the heuristic)
     else:
-        assert not any(key[0].startswith("rechunk-p2p") for key in x2.__dask_keys__())
+        assert all(key[0][0].startswith("rechunk-p2p") for key in x2.__dask_keys__())
 
     assert x2.chunks == new
     assert np.all(await c.compute(x2) == a)
 
 
+@pytest.mark.parametrize(
+    ["new", "expected_algorithm"],
+    [
+        # All-to-all rechunking defaults to P2P
+        (((1,) * 100, (100,)), "p2p"),
+        # Localized rechunking defaults to tasks
+        (((50, 50), (2,) * 50), "tasks"),
+        # Less local rechunking first defaults to tasks,
+        (((25, 25, 25, 25), (4,) * 25), "tasks"),
+        # then switches to p2p
+        (((10,) * 10, (10,) * 10), "p2p"),
+    ],
+)
+@gen_cluster(client=True)
+async def test_rechunk_heuristic(c, s, a, b, new, expected_algorithm):
+    a = np.random.default_rng().uniform(0, 1, 10000).reshape((100, 100))
+    x = da.from_array(a, chunks=(100, 1))
+    x2 = rechunk(x, chunks=new)
+    if expected_algorithm == "p2p":
+        assert all(key[0][0].startswith("rechunk-p2p") for key in x2.__dask_keys__())
+    else:
+        assert not any(
+            key[0][0].startswith("rechunk-p2p") for key in x2.__dask_keys__()
+        )
+
+    assert x2.chunks == new
+    assert np.all(await c.compute(x2) == a)
+
+
+@gen_cluster(client=True)
+async def test_cull_p2p_rechunk_independent_partitions(c, s, *ws):
+    a = np.random.default_rng().uniform(0, 1, 1000).reshape((10, 10, 10))
+    x = da.from_array(a, chunks=(1, 5, 1))
+    new = (5, 1, -1)
+    rechunked = rechunk(x, chunks=new, method="p2p")
+    (dsk,) = dask.optimize(rechunked)
+    culled = rechunked[:5, :2]
+    (dsk_culled,) = dask.optimize(culled)
+
+    # The culled graph requires only 1/2 of the input tasks
+    n_inputs = len(
+        [1 for key in dsk.dask.get_all_dependencies() if key[0].startswith("array-")]
+    )
+    n_culled_inputs = len(
+        [
+            1
+            for key in dsk_culled.dask.get_all_dependencies()
+            if key[0].startswith("array-")
+        ]
+    )
+    assert n_culled_inputs == n_inputs / 4
+    # The culled graph should also have less than 1/4 the tasks
+    assert len(dsk_culled.dask) < len(dsk.dask) / 4
+
+    assert np.all(await c.compute(culled) == a[:5, :2])
+
+
+@gen_cluster(client=True)
+async def test_cull_p2p_rechunking_single_chunk(c, s, *ws):
+    a = np.random.default_rng().uniform(0, 1, 1000).reshape((10, 10, 10))
+    x = da.from_array(a, chunks=(1, 5, 1))
+    new = (5, 1, -1)
+    rechunked = rechunk(x, chunks=new, method="p2p")
+    (dsk,) = dask.optimize(rechunked)
+    culled = rechunked[:5, 1:2]
+    (dsk_culled,) = dask.optimize(culled)
+
+    # The culled graph requires only 1/2 of the input tasks
+    n_inputs = len(
+        [1 for key in dsk.dask.get_all_dependencies() if key[0].startswith("array-")]
+    )
+
+    assert n_inputs > 0
+
+    n_culled_inputs = len(
+        [
+            1
+            for key in dsk_culled.dask.get_all_dependencies()
+            if key[0].startswith("array-")
+        ]
+    )
+    assert n_culled_inputs == n_inputs / 4
+    # The culled graph should also have less than 1/4 the tasks
+    assert len(dsk_culled.dask) < len(dsk.dask) / 4
+
+    assert np.all(await c.compute(culled) == a[:5, 1:2])
+
+
+@gen_cluster(client=True)
+async def test_cull_p2p_rechunk_overlapping_partitions(c, s, *ws):
+    a = np.random.default_rng().uniform(0, 1, 500).reshape((10, 10, 5))
+    x = da.from_array(a, chunks=(1, 5, 1))
+    new = (5, 3, -1)
+    rechunked = rechunk(x, chunks=new, method="p2p")
+    (dsk,) = dask.optimize(rechunked)
+    culled = rechunked[:5, :2]
+    (dsk_culled,) = dask.optimize(culled)
+
+    # The culled graph requires only 1/4 of the input tasks
+    n_inputs = len(
+        [1 for key in dsk.dask.get_all_dependencies() if key[0].startswith("array-")]
+    )
+    n_culled_inputs = len(
+        [
+            1
+            for key in dsk_culled.dask.get_all_dependencies()
+            if key[0].startswith("array-")
+        ]
+    )
+    assert n_culled_inputs == n_inputs / 4
+    # The culled graph should also have less than 1/4 the tasks
+    assert len(dsk_culled.dask) < len(dsk.dask) / 4
+
+    assert np.all(await c.compute(culled) == a[:5, :2])
+
+
 @pytest.mark.parametrize("disk", [True, False])
 @gen_cluster(client=True)
 async def test_rechunk_2d(c, s, *ws, disk):
@@ -214,7 +338,7 @@ async def test_rechunk_2d(c, s, *ws, disk):
     a = np.random.default_rng().uniform(0, 1, 300).reshape((10, 30))
     x = da.from_array(a, chunks=((1, 2, 3, 4), (5,) * 6))
     new = ((5, 5), (15,) * 2)
-    with dask.config.set({"distributed.p2p.disk": disk}):
+    with dask.config.set({"distributed.p2p.storage.disk": disk}):
         x2 = rechunk(x, chunks=new, method="p2p")
     assert x2.chunks == new
     assert np.all(await c.compute(x2) == a)
@@ -232,40 +356,14 @@ async def test_rechunk_4d(c, s, *ws, disk):
     old = ((5, 5),) * 4
     a = np.random.default_rng().uniform(0, 1, 10000).reshape((10,) * 4)
     x = da.from_array(a, chunks=old)
-    new = (
-        (10,),
-        (10,),
-        (10,),
-        (8, 2),
-    )  # This has been altered to return >1 output partition
-    with dask.config.set({"distributed.p2p.disk": disk}):
+    new = ((10,),) * 4
+    with dask.config.set({"distributed.p2p.storage.disk": disk}):
         x2 = rechunk(x, chunks=new, method="p2p")
     assert x2.chunks == new
     await c.compute(x2)
     assert np.all(await c.compute(x2) == a)
 
 
-@gen_cluster(client=True)
-async def test_rechunk_with_single_output_chunk_raises(c, s, *ws):
-    """See distributed#7816
-
-    See Also
-    --------
-    dask.array.tests.test_rechunk.test_rechunk_4d
-    """
-    old = ((5, 5),) * 4
-    a = np.random.default_rng().uniform(0, 1, 10000).reshape((10,) * 4)
-    x = da.from_array(a, chunks=old)
-    new = ((10,),) * 4
-    x2 = rechunk(x, chunks=new, method="p2p")
-    assert x2.chunks == new
-    # FIXME: distributed#7816
-    with raises_with_cause(
-        RuntimeError, "failed during transfer", RuntimeError, "Barrier task"
-    ):
-        await c.compute(x2)
-
-
 @gen_cluster(client=True)
 async def test_rechunk_expand(c, s, *ws):
     """
@@ -499,6 +597,7 @@ async def test_rechunk_same_fully_unknown(c, s, *ws):
     --------
     dask.array.tests.test_rechunk.test_rechunk_same_fully_unknown
     """
+    pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
     x = da.ones(shape=(10, 10), chunks=(5, 10))
     y = dd.from_array(x).values
@@ -517,6 +616,8 @@ async def test_rechunk_same_fully_unknown_floats(c, s, *ws):
     --------
     dask.array.tests.test_rechunk.test_rechunk_same_fully_unknown_floats
     """
+
+    pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
     x = da.ones(shape=(10, 10), chunks=(5, 10))
     y = dd.from_array(x).values
@@ -532,6 +633,7 @@ async def test_rechunk_same_partially_unknown(c, s, *ws):
     --------
     dask.array.tests.test_rechunk.test_rechunk_same_partially_unknown
     """
+    pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
     x = da.ones(shape=(10, 10), chunks=(5, 10))
     y = dd.from_array(x).values
@@ -589,8 +691,8 @@ async def test_rechunk_unknown_from_pandas(c, s, *ws):
     --------
     dask.array.tests.test_rechunk.test_rechunk_unknown_from_pandas
     """
-    dd = pytest.importorskip("dask.dataframe")
     pd = pytest.importorskip("pandas")
+    dd = pytest.importorskip("dask.dataframe")
 
     arr = np.random.default_rng().standard_normal((50, 10))
     x = dd.from_pandas(pd.DataFrame(arr), 2).values
@@ -611,6 +713,7 @@ async def test_rechunk_unknown_from_array(c, s, *ws):
     --------
     dask.array.tests.test_rechunk.test_rechunk_unknown_from_array
     """
+    pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
     x = dd.from_array(da.ones(shape=(4, 4), chunks=(2, 2))).values
     result = x.rechunk((None, 4), method="p2p")
@@ -644,6 +747,7 @@ async def test_rechunk_with_fully_unknown_dimension(c, s, *ws, x, chunks):
     --------
     dask.array.tests.test_rechunk.test_rechunk_with_fully_unknown_dimension
     """
+    pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
     y = dd.from_array(x).values
     result = y.rechunk(chunks, method="p2p")
@@ -686,6 +790,7 @@ async def test_rechunk_with_partially_unknown_dimension(c, s, *ws, x, chunks):
     --------
     dask.array.tests.test_rechunk.test_rechunk_with_partially_unknown_dimension
     """
+    pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
     y = dd.from_array(x).values
     z = da.concatenate([x, y])
@@ -711,6 +816,7 @@ async def test_rechunk_with_fully_unknown_dimension_explicit(c, s, *ws, new_chun
     --------
     dask.array.tests.test_rechunk.test_rechunk_with_fully_unknown_dimension_explicit
     """
+    pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
     x = da.ones(shape=(10, 10), chunks=(5, 2))
     y = dd.from_array(x).values
@@ -732,6 +838,7 @@ async def test_rechunk_unknown_raises(c, s, *ws):
     --------
     dask.array.tests.test_rechunk.test_rechunk_unknown_raises
     """
+    pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
 
     x = da.ones(shape=(10, 10), chunks=(5, 5))
@@ -754,8 +861,6 @@ async def test_rechunk_zero_dim(c, s, *ws):
     --------
     dask.array.tests.test_rechunk.test_rechunk_zero_dim
     """
-    da = pytest.importorskip("dask.array")
-
     x = da.ones((0, 10, 100), chunks=(0, 10, 10)).rechunk((0, 10, 50), method="p2p")
     assert len(await c.compute(x)) == 0
 
@@ -772,13 +877,13 @@ async def test_rechunk_empty_chunks(c, s, *ws):
     assert_eq(await c.compute(x), await c.compute(y))
 
 
-@pytest.mark.skip(reason="FIXME: We should avoid P2P in this case")
 @gen_cluster(client=True)
 async def test_rechunk_avoid_needless_chunking(c, s, *ws):
     x = da.ones(16, chunks=2)
     y = x.rechunk(8, method="p2p")
     dsk = y.__dask_graph__()
-    assert len(dsk) <= 8 + 2
+    # 8 inputs, 2 concatenations of small inputs, 2 outputs
+    assert len(dsk) <= 8 + 2 + 2
 
 
 @pytest.mark.parametrize(
@@ -1167,7 +1272,7 @@ async def test_preserve_writeable_flag(c, s, a, b):
     assert out.tolist() == [True, True]
 
 
-@gen_cluster(client=True, config={"distributed.p2p.disk": False})
+@gen_cluster(client=True, config={"distributed.p2p.storage.disk": False})
 async def test_rechunk_in_memory_shards_dont_share_buffer(c, s, a, b):
     """Test that, if two shards are sent in the same RPC call and they contribute to
     different output chunks, downstream tasks don't need to consume all output chunks in
@@ -1204,7 +1309,7 @@ async def test_rechunk_in_memory_shards_dont_share_buffer(c, s, a, b):
 
 
 @pytest.mark.parametrize("nworkers", [1, 2, 41, 50])
-def test_worker_for_homogeneous_distribution(nworkers):
+def test_pick_worker_homogeneous_distribution(nworkers):
     old = ((1, 2, 3, 4), (5,) * 6)
     new = ((5, 5), (12, 18))
     workers = [str(i) for i in range(nworkers)]
@@ -1216,3 +1321,218 @@ def test_worker_for_homogeneous_distribution(nworkers):
     assert sum(count.values()) > 0
     assert sum(count.values()) == len(list(spec.output_partitions))
     assert abs(max(count.values()) - min(count.values())) <= 1
+
+
+@pytest.mark.slow
+@gen_cluster(
+    client=True,
+    nthreads=[("", 1)] * 5,
+    # Future-proof: disable auto-rebalancing
+    config={"distributed.scheduler.active-memory-manager.start": False},
+)
+async def test_partial_rechunk_homogeneous_distribution(c, s, *workers):
+    # This rechunk operation can be split into 10 independent shuffles with 4 output
+    # chunks each. This is less than the number of workers, so we are at risk of
+    # choosing the same 4 output workers in each separate shuffle.
+    arr = da.random.random((40, 40), chunks=(4, 4))
+    arr = arr.rechunk((1, -1), method="p2p")
+    # Test that we are, in fact, triggering partial rechunks
+    assert sum(key_split(k) == "shuffle-barrier" for k in arr.__dask_graph__()) == 10
+
+    arr = await c.persist(arr)
+    # Don't count input and intermediate keys that have not been released yet
+    out_keys = set(flatten(arr.__dask_keys__()))
+    nchunks = [len(w.data.keys() & out_keys) for w in workers]
+    # There are 40 output chunks and 5 workers. Expect exactly 8 chunks per worker.
+    assert nchunks == [8, 8, 8, 8, 8]
+
+
+@gen_cluster(client=True, nthreads=[], config={"optimization.fuse.active": False})
+async def test_partial_rechunk_taskgroups(c, s):
+    """Regression test for https://github.com/dask/distributed/issues/8656"""
+    arr = da.random.random(
+        (10, 10),
+        chunks=(
+            (1,) * 10,
+            (2,) * 5,
+        ),
+    )
+    arr = arr.rechunk(
+        (
+            (2,) * 5,
+            (1,) * 10,
+        ),
+        method="p2p",
+    )
+
+    _ = c.compute(arr)
+    await async_poll_for(
+        lambda: any(
+            isinstance(task, str) and task.startswith("shuffle-barrier")
+            for task in s.tasks
+        ),
+        timeout=5,
+    )
+    assert len(s.task_groups) < 7
+
+
+@pytest.mark.parametrize(
+    ["old", "new", "expected"],
+    [
+        [((2, 2),), ((2, 2),), ((2, 2),)],
+        [((2, 2),), ((4,),), ((2, 2),)],
+        [((2, 2),), ((1, 1, 1, 1),), ((2, 2),)],
+        [((2, 2, 2),), ((1, 2, 2, 1),), ((1, 1, 1, 1, 1, 1),)],
+        [((1, np.nan),), ((1, np.nan),), ((1, np.nan),)],
+    ],
+)
+def test_calculate_prechunking_1d(old, new, expected):
+    actual = _calculate_prechunking(old, new, np.dtype, None)
+    assert actual == expected
+
+
+@pytest.mark.parametrize(
+    ["old", "new", "expected"],
+    [
+        [((2, 2), (3, 3)), ((2, 2), (3, 3)), ((2, 2), (3, 3))],
+        [((2, 2), (3, 3)), ((4,), (3, 3)), ((4,), (3, 3))],
+        [((2, 2), (3, 3)), ((1, 1, 1, 1), (3, 3)), ((2, 2), (3, 3))],
+        [
+            ((2, 2, 2), (3, 3, 3)),
+            ((1, 2, 2, 1), (2, 3, 4)),
+            ((1, 2, 2, 1), (2, 3, 4)),
+        ],
+        [((1, np.nan), (3, 3)), ((1, np.nan), (2, 2, 2)), ((1, np.nan), (2, 1, 1, 2))],
+        [((4,), (1, 1, 1)), ((1, 1, 1, 1), (3,)), ((4,), (3,))],
+    ],
+)
+def test_calculate_prechunking_2d(old, new, expected):
+    actual = _calculate_prechunking(old, new, np.dtype(np.int16), None)
+    assert actual == expected
+
+
+@pytest.mark.parametrize(
+    ["old", "new", "expected"],
+    [
+        (
+            ((2, 2), (1, 1, 1, 1), (1, 1, 1, 1)),
+            ((1, 1, 1, 1), (4,), (2, 2)),
+            ((2, 2), (4,), (1, 1, 1, 1)),
+        ),
+        (
+            ((2, 2), (1, 1, 1, 1), (1, 1, 1, 1)),
+            ((1, 1, 1, 1), (2, 2), (2, 2)),
+            ((2, 2), (2, 2), (2, 2)),
+        ),
+        (
+            ((2, 2), (1, 1, 1, 1), (1, 1, 1, 1)),
+            ((1, 1, 1, 1), (2, 2), (4,)),
+            ((2, 2), (2, 2), (2, 2)),
+        ),
+        (
+            ((1, 1, 1, 1), (1, 1, 1, 1), (2, 2)),
+            ((2, 2), (4,), (1, 1, 1, 1)),
+            ((2, 2), (2, 2), (2, 2)),
+        ),
+    ],
+)
+def test_calculate_prechunking_3d(old, new, expected):
+    with dask.config.set({"array.chunk-size": "16 B"}):
+        actual = _calculate_prechunking(old, new, np.dtype(np.int16), None)
+    assert actual == expected
+
+
+@pytest.mark.parametrize(
+    ["chunk_size", "expected"],
+    [
+        ("1 B", ((10,), (1,) * 10)),
+        ("20 B", ((10,), (1,) * 10)),
+        ("40 B", ((10,), (2, 2, 1, 2, 2, 1))),
+        ("100 B", ((10,), (5, 5))),
+    ],
+)
+def test_calculate_prechunking_concatenation(chunk_size, expected):
+    old = ((10,), (1,) * 10)
+    new = ((2,) * 5, (5, 5))
+    with dask.config.set({"array.chunk-size": chunk_size}):
+        actual = _calculate_prechunking(old, new, np.dtype(np.int16), None)
+    assert actual == expected
+
+
+def test_calculate_prechunking_does_not_concatenate_object_type():
+    old = ((10,), (1,) * 10)
+    new = ((2,) * 5, (5, 5))
+
+    # Ensure that int dtypes get concatenated
+    new = ((2,) * 5, (5, 5))
+    with dask.config.set({"array.chunk-size": "100 B"}):
+        actual = _calculate_prechunking(old, new, np.dtype(np.int16), None)
+    assert actual == ((10,), (5, 5))
+
+    # Ensure object dtype chunks do not get concatenated
+    with dask.config.set({"array.chunk-size": "100 B"}):
+        actual = _calculate_prechunking(old, new, np.dtype(object), None)
+    assert actual == old
+
+
+@pytest.mark.parametrize(
+    ["old", "new", "expected"],
+    [
+        [((2, 2), (3, 3)), ((4,), (3, 3)), ((2, 2), (3, 3))],
+        [
+            ((2, 2, 2), (3, 3, 3)),
+            ((1, 2, 2, 1), (2, 3, 4)),
+            ((1, 1, 1, 1, 1, 1), (2, 1, 2, 1, 3)),
+        ],
+        [((4,), (1, 1, 1)), ((1, 1, 1, 1), (3,)), ((4,), (1, 1, 1))],
+    ],
+)
+def test_calculate_prechunking_splitting(old, new, expected):
+    # _calculate_prechunking does not concatenate on object
+    actual = _calculate_prechunking(old, new, np.dtype(object), None)
+    assert actual == expected
+
+
+@gen_cluster(client=True, nthreads=[("", 1)] * 4, config={"array.chunk-size": "1 B"})
+async def test_homogeneously_schedule_unpack(c, s, *ws):
+    class SchedulingTrackerPlugin(SchedulerPlugin):
+        async def start(self, scheduler):
+            self.scheduler = scheduler
+            self.counts = defaultdict(int)
+            self.seen = set()
+
+        def transition(self, key, start, finish, *args, stimulus_id, **kwargs):
+            if key in self.seen:
+                return
+
+            if not isinstance(key, tuple) or not isinstance(key[0], str):
+                return
+
+            if not key[0].startswith("rechunk-p2p"):
+                return
+
+            if start != "waiting" or finish != "processing":
+                return
+
+            self.seen.add(key)
+            self.counts[self.scheduler.tasks[key].processing_on.address] += 1
+
+    await c.register_plugin(SchedulingTrackerPlugin(), name="tracker")
+    res = da.random.random((100, 100), chunks=(1, -1)).rechunk((-1, 1))
+    await c.compute(res)
+    counts = s.plugins["tracker"].counts
+    min_count = min(counts.values())
+    max_count = max(counts.values())
+    assert min_count >= max_count, counts
+
+
+@pytest.mark.parametrize("method", ["tasks", "p2p"])
+@gen_cluster(client=True)
+async def test_rechunk_datetime(c, s, *ws, method):
+    pd = pytest.importorskip("pandas")
+
+    x = pd.date_range("2005-01-01", "2005-01-10").to_numpy(dtype="datetime64[ns]")
+    dx = da.from_array(x, chunks=10)
+    result = dx.rechunk(2, method=method)
+    result = await c.compute(result)
+    np.testing.assert_array_equal(x, result)
