@@ -39,6 +39,7 @@ from distributed.worker_state_machine import (
     SecedeEvent,
     TaskFinishedMsg,
     UpdateDataEvent,
+    WorkerState,
 )
 
 
@@ -169,7 +170,11 @@ async def test_flight_to_executing_via_cancelled_resumed(c, s, b):
         with lock:
             return x + 1
 
-    async with BrokenWorker(s.address) as a:
+    async with BrokenWorker(
+        s.address,
+        # heartbeat will close a worker after remove_worker(close=False)
+        heartbeat_interval="100s",
+    ) as a:
         await c.wait_for_workers(2)
         fut1 = c.submit(
             blockable_compute,
@@ -267,7 +272,11 @@ def test_flight_cancelled_error(ws):
 
 @gen_cluster(client=True, nthreads=[("", 1)])
 async def test_in_flight_lost_after_resumed(c, s, b):
-    async with BlockedGetData(s.address) as a:
+    async with BlockedGetData(
+        s.address,
+        # heartbeat will close a worker after remove_worker(close=False)
+        heartbeat_interval="100s",
+    ) as a:
         fut1 = c.submit(inc, 1, workers=[a.address], key="fut1")
         # Ensure fut1 is in memory but block any further execution afterwards to
         # ensure we control when the recomputation happens
@@ -282,7 +291,7 @@ async def test_in_flight_lost_after_resumed(c, s, b):
         s.set_restrictions({fut1.key: [a.address, b.address]})
         # It is removed, i.e. get_data is guaranteed to fail and f1 is scheduled
         # to be recomputed on B
-        await s.remove_worker(a.address, stimulus_id="foo", close=False, safe=True)
+        await s.remove_worker(a.address, stimulus_id="foo", close=False, expected=True)
 
         await wait_for_state(fut1.key, "resumed", b, interval=0)
 
@@ -817,8 +826,11 @@ async def test_cancelled_task_error_rejected(c, s, a, b):
 
 
 @pytest.mark.parametrize("intermediate_state", ["resumed", "cancelled"])
-@pytest.mark.parametrize("close_worker", [False, True])
-@gen_cluster(client=True, config={"distributed.comm.timeouts.connect": "500ms"})
+@pytest.mark.parametrize("close_worker", [True])
+@gen_cluster(
+    client=True,
+    config={"distributed.comm.timeouts.connect": "500ms"},
+)
 async def test_deadlock_cancelled_after_inflight_before_gather_from_worker(
     c, s, a, x, intermediate_state, close_worker
 ):
@@ -831,10 +843,34 @@ async def test_deadlock_cancelled_after_inflight_before_gather_from_worker(
     fut2 = c.submit(sum, [fut1, fut1B], workers=[x.address], key="f2")
     await fut2
 
-    async with BlockedGatherDep(s.address, name="b") as b:
+    class InstrumentedWorkerState(WorkerState):
+        def __init__(self, *args, **kwargs):
+            super().__init__(*args, **kwargs)
+            self.fut2_in_flight = asyncio.Event()
+            self.fut2_in_intermediate = asyncio.Event()
+
+        def _transition(self, ts, finish, *args, **kwargs):
+            def _verify_state(finish):
+                if ts.key == fut2.key:
+                    if isinstance(finish, tuple) and finish[0] == "flight":
+                        self.fut2_in_flight.set()
+                    if self.fut2_in_flight.is_set() and finish == intermediate_state:
+                        self.fut2_in_intermediate.set()
+
+            # The expected state might be either the requested one or the
+            # actual, final state
+            _verify_state(finish)
+            try:
+                return super()._transition(ts, finish, *args, **kwargs)
+            finally:
+                _verify_state(ts.state)
+
+    async with BlockedGatherDep(
+        s.address, name="b", WorkerStateClass=InstrumentedWorkerState
+    ) as b:
         fut3 = c.submit(inc, fut2, workers=[b.address], key="f3")
 
-        await wait_for_state(fut2.key, "flight", b)
+        await b.state.fut2_in_flight.wait()
 
         s.set_restrictions(worker={fut1B.key: a.address, fut2.key: b.address})
 
@@ -842,12 +878,12 @@ async def test_deadlock_cancelled_after_inflight_before_gather_from_worker(
 
         await s.remove_worker(
             address=x.address,
-            safe=True,
+            expected=True,
             close=close_worker,
             stimulus_id="remove-worker",
         )
 
-        await wait_for_state(fut2.key, intermediate_state, b, interval=0)
+        await b.state.fut2_in_intermediate.wait()
 
         b.block_gather_dep.set()
         await fut3
