@@ -2,12 +2,12 @@ from __future__ import annotations
 
 import asyncio
 import contextlib
+import logging
 import os
 import random
 import socket
 import sys
 import threading
-import time as timemod
 import weakref
 from unittest import mock
 
@@ -17,13 +17,12 @@ from tornado.ioloop import IOLoop
 import dask
 
 from distributed.batched import BatchedSend
-from distributed.comm.core import CommClosedError
+from distributed.comm.core import CommClosedError, FatalCommClosedError
 from distributed.comm.registry import backends
 from distributed.comm.tcp import TCPBackend, TCPListener
 from distributed.core import (
-    AsyncTaskGroup,
-    AsyncTaskGroupClosedError,
     ConnectionPool,
+    RPCClosed,
     Server,
     Status,
     _expects_comm,
@@ -83,156 +82,6 @@ def echo_no_serialize(comm, x):
     return {"result": x}
 
 
-def test_async_task_group_initialization():
-    group = AsyncTaskGroup()
-    assert not group.closed
-    assert len(group) == 0
-
-
-async def _wait_for_n_loop_cycles(n):
-    for _ in range(n):
-        await asyncio.sleep(0)
-
-
-@gen_test()
-async def test_async_task_group_call_soon_executes_task_in_background():
-    group = AsyncTaskGroup()
-    ev = asyncio.Event()
-    flag = False
-
-    async def set_flag():
-        nonlocal flag
-        await ev.wait()
-        flag = True
-
-    assert group.call_soon(set_flag) is None
-    assert len(group) == 1
-    ev.set()
-    await _wait_for_n_loop_cycles(2)
-    assert len(group) == 0
-    assert flag
-
-
-@gen_test()
-async def test_async_task_group_call_later_executes_delayed_task_in_background():
-    group = AsyncTaskGroup()
-    ev = asyncio.Event()
-
-    start = timemod.monotonic()
-    assert group.call_later(1, ev.set) is None
-    assert len(group) == 1
-    await ev.wait()
-    end = timemod.monotonic()
-    # the task must be removed in exactly 1 event loop cycle
-    await _wait_for_n_loop_cycles(2)
-    assert len(group) == 0
-    assert end - start > 1 - timemod.get_clock_info("monotonic").resolution
-
-
-def test_async_task_group_close_closes():
-    group = AsyncTaskGroup()
-    group.close()
-    assert group.closed
-
-    # Test idempotency
-    group.close()
-    assert group.closed
-
-
-@gen_test()
-async def test_async_task_group_close_does_not_cancel_existing_tasks():
-    group = AsyncTaskGroup()
-
-    ev = asyncio.Event()
-    flag = False
-
-    async def set_flag():
-        nonlocal flag
-        await ev.wait()
-        flag = True
-        return None
-
-    assert group.call_soon(set_flag) is None
-
-    group.close()
-
-    assert len(group) == 1
-
-    ev.set()
-    await _wait_for_n_loop_cycles(2)
-    assert len(group) == 0
-
-
-@gen_test()
-async def test_async_task_group_close_prohibits_new_tasks():
-    group = AsyncTaskGroup()
-    group.close()
-
-    ev = asyncio.Event()
-    flag = False
-
-    async def set_flag():
-        nonlocal flag
-        await ev.wait()
-        flag = True
-        return True
-
-    with pytest.raises(AsyncTaskGroupClosedError):
-        group.call_soon(set_flag)
-    assert len(group) == 0
-
-    with pytest.raises(AsyncTaskGroupClosedError):
-        group.call_later(1, set_flag)
-    assert len(group) == 0
-
-    await asyncio.sleep(0.01)
-    assert not flag
-
-
-@gen_test()
-async def test_async_task_group_stop_disallows_shutdown():
-    group = AsyncTaskGroup()
-
-    task = None
-
-    async def set_flag():
-        nonlocal task
-        task = asyncio.current_task()
-
-    assert group.call_soon(set_flag) is None
-    assert len(group) == 1
-    # tasks are not given a grace period, and are not even allowed to start
-    # if the group is closed immediately
-    await group.stop()
-    assert task is None
-
-
-@gen_test()
-async def test_async_task_group_stop_cancels_long_running():
-    group = AsyncTaskGroup()
-
-    task = None
-    flag = False
-    started = asyncio.Event()
-
-    async def set_flag():
-        nonlocal task
-        task = asyncio.current_task()
-        started.set()
-        await asyncio.sleep(10)
-        nonlocal flag
-        flag = True
-        return True
-
-    assert group.call_soon(set_flag) is None
-    assert len(group) == 1
-    await started.wait()
-    await group.stop()
-    assert task
-    assert task.cancelled()
-    assert not flag
-
-
 @gen_test()
 async def test_server_status_is_always_enum():
     """Assignments with strings is forbidden"""
@@ -858,6 +707,47 @@ async def test_connection_pool_outside_cancellation(monkeypatch):
         assert all(t.cancelled() for t in tasks)
 
 
+@gen_test()
+async def test_connection_pool_catch_all_cancellederrors(monkeypatch):
+    from distributed.comm.registry import backends
+    from distributed.comm.tcp import TCPBackend, TCPConnector
+
+    in_connect = asyncio.Event()
+    block_connect = asyncio.Event()
+
+    class BlockedConnector(TCPConnector):
+        async def connect(self, address, deserialize, **connection_args):
+            # This is extremely artificial and assumes that something further
+            # down in the stack would block a cancellation. We want to make sure
+            # that our ConnectionPool closes regardless of this.
+            in_connect.set()
+            try:
+                await block_connect.wait()
+            except asyncio.CancelledError:
+                await asyncio.sleep(30)
+                raise
+            raise FatalCommClosedError()
+
+    class BlockedConnectBackend(TCPBackend):
+        _connector_class = BlockedConnector
+
+    monkeypatch.setitem(backends, "tcp", BlockedConnectBackend())
+
+    async with Server({}) as server:
+        await server.listen("tcp://")
+        pool = await ConnectionPool(limit=2)
+        pool._connecting_close_timeout = 0
+
+        t = asyncio.create_task(pool.connect(server.address))
+
+        await in_connect.wait()
+        while not pool._connecting_count:
+            await asyncio.sleep(0.1)
+        with captured_logger("distributed.core") as sio:
+            await pool.close()
+        assert "Pending connections refuse to cancel" in sio.getvalue()
+
+
 @gen_test()
 async def test_remove_cancels_connect_attempts():
     loop = asyncio.get_running_loop()
@@ -1075,6 +965,20 @@ async def test_rpc_serialization():
             assert result == {"result": inc}
 
 
+@gen_test()
+async def test_rpc_closed_exception():
+    async with Server({"echo": echo_serialize}) as server:
+        await server.listen("tcp://")
+
+        async with rpc(server.address, serializers=["msgpack"]) as r:
+            r.status = Status.closed
+            with pytest.raises(
+                RPCClosed,
+                match="Exception while trying to call remote method .* before comm was established.",
+            ):
+                await r.__getattr__("foo")()
+
+
 @gen_cluster()
 async def test_thread_id(s, a, b):
     assert s.thread_id == a.thread_id == b.thread_id == threading.get_ident()
@@ -1189,9 +1093,12 @@ async def test_server_comms_mark_active_handlers():
     ensure this is properly reflected and released. The sentinel for
     "open comm but no active handler" is `None`
     """
+    in_handler = asyncio.Event()
+    unblock_handler = asyncio.Event()
 
     async def long_handler(comm):
-        await asyncio.sleep(0.2)
+        in_handler.set()
+        await unblock_handler.wait()
         return "done"
 
     async with Server({"wait": long_handler}) as server:
@@ -1200,9 +1107,9 @@ async def test_server_comms_mark_active_handlers():
 
         comm = await connect(server.address)
         await comm.write({"op": "wait"})
-        while not server._comms:
-            await asyncio.sleep(0.05)
+        await in_handler.wait()
         assert set(server._comms.values()) == {"wait"}
+        unblock_handler.set()
 
         assert server.incoming_comms_open == 1
         assert server.incoming_comms_active == 1
@@ -1243,9 +1150,12 @@ async def test_server_comms_mark_active_handlers():
 
         async with Server({}) as server2:
             rpc_ = server2.rpc(server.address)
+            in_handler.clear()
+            unblock_handler.clear()
             task = asyncio.create_task(rpc_.wait())
-            while not server.incoming_comms_active:
-                await asyncio.sleep(0.1)
+
+            await in_handler.wait()
+
             assert server.incoming_comms_active == 1
             assert server.incoming_comms_open == 1
             assert server.outgoing_comms_active == 0
@@ -1257,6 +1167,7 @@ async def test_server_comms_mark_active_handlers():
             assert server2.outgoing_comms_open == 1
             validate_dict(server)
 
+            unblock_handler.set()
             await task
             assert server.incoming_comms_active == 0
             assert server.incoming_comms_open == 1
@@ -1336,38 +1247,27 @@ async def test_close_grace_period_for_handlers():
 
 def test_expects_comm():
     class A:
-        def empty(self):
-            ...
+        def empty(self): ...
 
-        def one_arg(self, arg):
-            ...
+        def one_arg(self, arg): ...
 
-        def comm_arg(self, comm):
-            ...
+        def comm_arg(self, comm): ...
 
-        def stream_arg(self, stream):
-            ...
+        def stream_arg(self, stream): ...
 
-        def two_arg(self, arg, other):
-            ...
+        def two_arg(self, arg, other): ...
 
-        def comm_arg_other(self, comm, other):
-            ...
+        def comm_arg_other(self, comm, other): ...
 
-        def stream_arg_other(self, stream, other):
-            ...
+        def stream_arg_other(self, stream, other): ...
 
-        def arg_kwarg(self, arg, other=None):
-            ...
+        def arg_kwarg(self, arg, other=None): ...
 
-        def comm_posarg_only(self, comm, /, other):
-            ...
+        def comm_posarg_only(self, comm, /, other): ...
 
-        def comm_not_leading_position(self, other, comm):
-            ...
+        def comm_not_leading_position(self, other, comm): ...
 
-        def stream_not_leading_position(self, other, stream):
-            ...
+        def stream_not_leading_position(self, other, stream): ...
 
     expected_warning = "first argument of a RPC handler `stream` is deprecated"
 
@@ -1481,3 +1381,27 @@ async def test_messages_are_ordered_raw():
             assert ledger == list(range(n))
         finally:
             await comm.close()
+
+
+@pytest.mark.slow
+@gen_test(timeout=180)
+async def test_large_payload(caplog):
+    """See also: protocol/tests/test_protocol.py::test_large_payload"""
+    critical_size = 2**31 + 1  # >2 GiB
+    data = b"0" * critical_size
+
+    async with Server({"echo": echo_serialize}) as server:
+        await server.listen(0)
+        comm = await connect(server.address)
+
+        # FIXME https://github.com/dask/distributed/issues/8465
+        # At debug level, messages are dumped into the log. By default, pytest captures
+        # all logs, which would make this test extremely expensive to run.
+        with caplog.at_level(logging.INFO, logger="distributed.core"):
+            # Note: if we wrap data in to_serialize, it will be sent as a buffer, which
+            # is not encoded by msgpack.
+            await comm.write({"op": "echo", "x": data})
+            response = await comm.read()
+
+        assert response["result"] == data
+        await comm.close()
