@@ -36,7 +36,6 @@ from distributed.utils import (
     RateLimiterFilter,
     TimeoutError,
     TupleComparable,
-    _maybe_complex,
     ensure_ip,
     ensure_memoryview,
     format_dashboard_link,
@@ -234,15 +233,6 @@ def test_get_traceback():
         assert type(tb).__name__ == "traceback"
 
 
-def test_maybe_complex():
-    assert not _maybe_complex(1)
-    assert not _maybe_complex("x")
-    assert _maybe_complex((inc, 1))
-    assert _maybe_complex([(inc, 1)])
-    assert _maybe_complex([(inc, 1)])
-    assert _maybe_complex({"x": (inc, 1)})
-
-
 def test_read_block():
     delimiter = b"\n"
     data = delimiter.join([b"123", b"456", b"789"])
@@ -879,7 +869,10 @@ async def test_log_errors():
         pass
 
     # Use the logger of the caller module
-    with captured_logger("test_utils") as caplog:
+    with (
+        captured_logger("test_utils") as caplog,
+        captured_logger("distributed.utils") as caplog2,
+    ):
         # Context manager
         with log_errors():
             pass
@@ -906,6 +899,8 @@ async def test_log_errors():
             return 123
 
         assert _() == 123
+        with pytest.raises(TypeError):
+            _(bad=1)
 
         @log_errors
         def _():
@@ -1001,6 +996,7 @@ async def test_log_errors():
         "err7",
         "err8",
     ]
+    assert "got an unexpected keyword argument 'bad'" in caplog2.getvalue()
 
 
 @pytest.mark.parametrize("unroll_stack,logger_name", [(0, "test_utils"), (1, "a.b.c")])
