@@ -20,6 +20,7 @@ import traceback
 import warnings
 import weakref
 import xml.etree.ElementTree
+from asyncio import Event as LateLoopEvent
 from asyncio import TimeoutError
 from collections import deque
 from collections.abc import (
@@ -49,9 +50,9 @@ from typing import TYPE_CHECKING
 from typing import Any as AnyType
 from typing import ClassVar, TypeVar, overload
 
-import click
 import psutil
 import tblib.pickling_support
+from tornado import escape
 
 from distributed.compatibility import asyncio_run
 from distributed.config import get_loop_factory
@@ -66,7 +67,6 @@ from tornado import gen
 from tornado.ioloop import IOLoop
 
 import dask
-from dask import istask
 from dask.utils import ensure_bytes as _ensure_bytes
 from dask.utils import key_split
 from dask.utils import parse_timedelta as _parse_timedelta
@@ -170,12 +170,20 @@ def get_fileno_limit():
 
 @toolz.memoize
 def _get_ip(host, port, family):
+    def hostname_fallback():
+        addr_info = socket.getaddrinfo(
+            socket.gethostname(), port, family, socket.SOCK_DGRAM, socket.IPPROTO_UDP
+        )[0]
+        return addr_info[4][0]
+
     # By using a UDP socket, we don't actually try to connect but
     # simply select the local address through which *host* is reachable.
     sock = socket.socket(family, socket.SOCK_DGRAM)
     try:
         sock.connect((host, port))
         ip = sock.getsockname()[0]
+        if ip == "0.0.0.0":
+            return hostname_fallback()
         return ip
     except OSError as e:
         warnings.warn(
@@ -183,10 +191,7 @@ def _get_ip(host, port, family):
             "reaching %r, defaulting to hostname: %s" % (host, e),
             RuntimeWarning,
         )
-        addr_info = socket.getaddrinfo(
-            socket.gethostname(), port, family, socket.SOCK_DGRAM, socket.IPPROTO_UDP
-        )[0]
-        return addr_info[4][0]
+        return hostname_fallback()
     finally:
         sock.close()
 
@@ -436,36 +441,6 @@ def sync(
         return result
 
 
-if sys.version_info >= (3, 10):
-    from asyncio import Event as LateLoopEvent
-else:
-    # In python 3.10 asyncio.Lock and other primitives no longer support
-    # passing a loop kwarg to bind to a loop running in another thread
-    # e.g. calling from Client(asynchronous=False). Instead the loop is bound
-    # as late as possible: when calling any methods that wait on or wake
-    # Future instances. See: https://bugs.python.org/issue42392
-    class LateLoopEvent:
-        _event: asyncio.Event | None
-
-        def __init__(self) -> None:
-            self._event = None
-
-        def set(self) -> None:
-            if self._event is None:
-                self._event = asyncio.Event()
-
-            self._event.set()
-
-        def is_set(self) -> bool:
-            return self._event is not None and self._event.is_set()
-
-        async def wait(self) -> bool:
-            if self._event is None:
-                self._event = asyncio.Event()
-
-            return await self._event.wait()
-
-
 class _CollectErrorThread:
     def __init__(self, target: Callable[[], None], daemon: bool, name: str):
         self._exception: BaseException | None = None
@@ -473,7 +448,7 @@ class _CollectErrorThread:
         def wrapper() -> None:
             try:
                 target()
-            except BaseException as e:
+            except BaseException as e:  # noqa: B036
                 self._exception = e
 
         self._thread = thread = threading.Thread(
@@ -747,13 +722,11 @@ def key_split_group(x: object) -> str:
 
 
 @overload
-def log_errors(func: Callable[P, T], /) -> Callable[P, T]:
-    ...
+def log_errors(func: Callable[P, T], /) -> Callable[P, T]: ...
 
 
 @overload
-def log_errors(*, pdb: bool = False, unroll_stack: int = 1) -> _LogErrors:
-    ...
+def log_errors(*, pdb: bool = False, unroll_stack: int = 1) -> _LogErrors: ...
 
 
 def log_errors(func=None, /, *, pdb=False, unroll_stack=0):
@@ -801,7 +774,7 @@ def _getmodulename_with_path(fname: str) -> str:
     except KeyError:
         pass
 
-    for modname, mod in sys.modules.items():
+    for modname, mod in sys.modules.copy().items():
         fname2 = getattr(mod, "__file__", None)
         if fname2:
             _getmodulename_with_path_map[fname2] = modname
@@ -849,7 +822,7 @@ class _LogErrors:
             return
 
         stack = traceback.extract_tb(tb)
-        frame = stack[self.unroll_stack]
+        frame = stack[min(self.unroll_stack, len(stack) - 1)]
         modname = _getmodulename_with_path(frame.filename)
 
         try:
@@ -889,7 +862,7 @@ def silence_logging(level, root="distributed"):
 @contextlib.contextmanager
 def silence_logging_cmgr(
     level: str | int, root: str = "distributed"
-) -> Generator[None, None, None]:
+) -> Generator[None]:
     """
     Temporarily change all StreamHandlers for the given logger to the given level
     """
@@ -948,6 +921,7 @@ def get_traceback():
         os.path.join("distributed", "scheduler"),
         os.path.join("tornado", "gen.py"),
         os.path.join("concurrent", "futures"),
+        os.path.join("dask", "_task_spec"),
     ]
     while exc_traceback and any(
         b in exc_traceback.tb_frame.f_code.co_filename for b in bad
@@ -967,17 +941,6 @@ def truncate_exception(e, n=10000):
         return e
 
 
-def _maybe_complex(task):
-    """Possibly contains a nested task"""
-    return (
-        istask(task)
-        or type(task) is list
-        and any(map(_maybe_complex, task))
-        or type(task) is dict
-        and any(map(_maybe_complex, task.values()))
-    )
-
-
 def seek_delimiter(file, delimiter, blocksize):
     """Seek current file to next byte after a delimiter bytestring
 
@@ -1268,6 +1231,10 @@ def has_keyword(func, keyword):
 
 @functools.lru_cache(1000)
 def command_has_keyword(cmd, k):
+    # Click is a relatively expensive import
+    # That hurts startup time a little
+    import click
+
     if cmd is not None:
         if isinstance(cmd, str):
             try:
@@ -1825,68 +1792,76 @@ def recursive_to_dict(
         tok.var.reset(tok)
 
 
-def is_python_shutting_down() -> bool:
-    """Is the interpreter shutting down now?
-
-    This is a variant of ``sys.is_finalizing`` which can return True inside the ``__del__``
-    method of classes defined inside the distributed package.
-    """
-    # This import must remain local for the global variable to be
-    # properly evaluated
-    from distributed import _python_shutting_down
-
-    return _python_shutting_down
-
-
 class Deadline:
     """Utility class tracking a deadline and the progress toward it"""
 
-    #: Expiry time of the deadline in seconds since the epoch
+    #: Expiry time of the deadline in seconds since the start of the monotonic timer
     #: or None if the deadline never expires
-    expires_at: float | None
+    expires_at_mono: float | None
     #: Seconds since the epoch when the deadline was created
+    started_at_mono: float
+    #: Seconds since the start of the monotonic timer when the deadline was created
     started_at: float
 
+    __slots__ = tuple(__annotations__)
+
     def __init__(self, expires_at: float | None = None):
-        self.expires_at = expires_at
         self.started_at = time()
+        self.started_at_mono = monotonic()
+        if expires_at is not None:
+            self.expires_at_mono = expires_at - self.started_at + self.started_at_mono
+        else:
+            self.expires_at_mono = None
 
     @classmethod
     def after(cls, duration: float | None = None) -> Deadline:
         """Create a new ``Deadline`` that expires in ``duration`` seconds
-        or never if ``duration`` is None"""
-        started_at = time()
-        expires_at = duration + started_at if duration is not None else duration
-        deadline = cls(expires_at)
-        deadline.started_at = started_at
-        return deadline
+        or never if ``duration`` is None
+        """
+        inst = cls()
+        if duration is not None:
+            inst.expires_at_mono = inst.started_at_mono + duration
+        return inst
+
+    @property
+    def expires_at(self) -> float | None:
+        """Expiry time of the deadline in seconds since the unix epoch
+        or None if the deadline never expires.
+
+        Note that this can change over time if the wall clock is adjusted by the OS.
+        """
+        if (exp := self.expires_at_mono) is None:
+            return None
+        return exp - monotonic() + time()
 
     @property
     def duration(self) -> float | None:
         """Seconds between the creation and expiration time of the deadline
-        if the deadline expires, None otherwise"""
-        if self.expires_at is None:
+        if the deadline expires, None otherwise
+        """
+        if (exp := self.expires_at_mono) is None:
             return None
-        return self.expires_at - self.started_at
+        return exp - self.started_at_mono
 
     @property
     def expires(self) -> bool:
         """Whether the deadline ever expires"""
-        return self.expires_at is not None
+        return self.expires_at_mono is not None
 
     @property
     def elapsed(self) -> float:
         """Seconds that elapsed since the deadline was created"""
-        return time() - self.started_at
+        return monotonic() - self.started_at_mono
 
     @property
     def remaining(self) -> float | None:
         """Seconds remaining until the deadline expires if an expiry time is set,
-        None otherwise"""
-        if self.expires_at is None:
+        None otherwise
+        """
+        if (exp := self.expires_at_mono) is None:
             return None
         else:
-            return max(0, self.expires_at - time())
+            return max(0, exp - monotonic())
 
     @property
     def expired(self) -> bool:
@@ -1973,3 +1948,11 @@ class TupleComparable:
 
     def __lt__(self, other):
         return self.obj < other.obj
+
+
+@functools.cache
+def url_escape(url, *args, **kwargs):
+    """
+    Escape a URL path segment. Cache results for better performance.
+    """
+    return escape.url_escape(url, *args, **kwargs)
